@@ -1,14 +1,11 @@
-#ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/paint.c,v 1.10 1999/06/02 01:00:43 unammx Exp $";
-#endif
 /*****
 * paint.c : XmHTML rendering routines
 *
-* This file Version	$Revision: 1.10 $
+* This file Version	$Revision: 1.11 $
 *
 * Creation date:		Fri Dec  6 19:50:20 GMT+0100 1996
-* Last modification: 	$Date: 1999/06/02 01:00:43 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1999/07/29 01:26:29 $
+* By:					$Author: sopwith $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -35,6 +32,11 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/paint.
 /*****
 * ChangeLog 
 * $Log: paint.c,v $
+* Revision 1.11  1999/07/29 01:26:29  sopwith
+*
+*
+* Fix all warnings.
+*
 * Revision 1.10  1999/06/02 01:00:43  unammx
 *
 * 1999-06-01  Akira Higuchi <a-higuti@math.sci.hokudai.ac.jp>
@@ -368,7 +370,6 @@ DrawText(XmHTMLWidget html, XmHTMLObjectTableElement data)
 {
 	int width, ys, xs, nwords = data->n_words;
 	XmHTMLWord *words = data->words;
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window (html->html.work_area);
 	TGC gc = html->html.gc;
 	register int i;
@@ -480,7 +481,6 @@ static void
 DrawAnchor(XmHTMLWidget html, XmHTMLObjectTableElement data)
 {
 	int x, xs, y, ys, width, start, nwords = 0;
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
 	TGC gc = html->html.gc;
 	XmHTMLfont *font;
@@ -809,7 +809,6 @@ static void
 DrawImageAnchor(XmHTMLWidget html, XmHTMLObjectTableElement data)
 {
 	int x, y, width, height;
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
 	TGC gc = html->html.gc;
 
@@ -1050,9 +1049,6 @@ static void
 DrawFrame(XmHTMLWidget html, XmHTMLImage *image, int xs, int ys)
 {
 	int idx, width = 0, height = 0, fx, fy;
-	unsigned long valuemask;
-	XGCValues values;
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
 	TGC gc = html->html.gc;
 
@@ -1414,8 +1410,6 @@ _XmHTMLDrawImage(XmHTMLWidget html, XmHTMLObjectTableElement data, int y_offset,
 {
 	int xs, ys;
 	XmHTMLImage *image;
-	unsigned long valuemask;
-	XGCValues values;
 	Display *dpy;
 	TWindow win;
 	TGC gc;
@@ -1565,7 +1559,6 @@ static void
 DrawRule(XmHTMLWidget html, XmHTMLObjectTableElement data)
 {
 	int dy;
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
 	TGC gc;
 	int xs, ys;
@@ -1631,7 +1624,7 @@ DrawRule(XmHTMLWidget html, XmHTMLObjectTableElement data)
 		{
 			if (data->fg != html->html.body_fg)
 				_XmHTMLRecomputeShadowColors(html, data->fg);
-			Toolkit_Draw_Shadows(html,Toolkit_StyleGC_TopShadow(html),
+			Toolkit_Draw_Shadows(html, Toolkit_StyleGC_TopShadow(html),
 					     Toolkit_StyleGC_BottomShadow(html), xs, ys+dy,
 					     data->width, data->height, 1, XmSHADOW_IN);
 			if (data->fg != html->html.body_fg)
@@ -1666,7 +1659,6 @@ DrawRule(XmHTMLWidget html, XmHTMLObjectTableElement data)
 static void
 DrawBullet(XmHTMLWidget html, XmHTMLObjectTableElement data)
 {
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
 	TGC gc = html->html.gc;
 	int ys, xs;
@@ -1711,7 +1703,6 @@ static void
 DrawAnchorButton(XmHTMLWidget html, int x, int y, Dimension width,
 	Dimension height, TGC top_shadow_GC, TGC bottom_shadow_GC)
 {
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
 
 	/* top & left border */
@@ -1874,7 +1865,6 @@ static void
 DrawCellFrame(XmHTMLWidget html, TableCell *cell)
 {
 	XmHTMLObjectTableElement data = cell->owner;
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
  	TGC gc;
  	int xs, ys, xoff, yoff, width, height;
@@ -1957,8 +1947,6 @@ DrawCellFrame(XmHTMLWidget html, TableCell *cell)
 	/* Do we have a background image? */
 	if(cell->properties->bg_image != NULL)
 	{
-		unsigned long valuemask;
-		XGCValues values;
 		int tile_width, tile_height, x_dist, y_dist;
 		int ntiles_x, ntiles_y, tsx, tsy;
 		int x_offset, y_offset;
@@ -2031,7 +2019,6 @@ DrawCellFrame(XmHTMLWidget html, TableCell *cell)
 static void
 DrawTableBorder(XmHTMLWidget html, XmHTMLTable *table)
 {
-	Display *dpy = Toolkit_Display(html->html.work_area);
 	TWindow win = Toolkit_Widget_Window(html->html.work_area);
 	TGC gc;
 	int xs, ys;
