@@ -1362,11 +1362,11 @@ void vt_free_match_blocks(struct _vtx *vx)
     b = m->blocks;
     while (b) {
       n = b->next;
-      free(b);
+      g_free(b);
       b=n;
     }
     mn = m->next;
-    free(m->matchstr);
+    g_free(m->matchstr);
     m = mn;
   }
   vx->matches = 0;
@@ -1449,8 +1449,8 @@ void vt_getmatches(struct _vtx *vx)
 	/* check this regex, for multiple matches ... */
 	out = line;
 	while ((out<outend) && regexec(&mw->preg, out, 1, pmatch, 0)==0) {
-	  int start = pmatch[0].rm_so;
-	  int end = pmatch[0].rm_eo;
+	  int start = pmatch[0].rm_so + (out-line);
+	  int end = pmatch[0].rm_eo + (out-line);
 	  struct vt_match_block *b;
 	  struct vt_match *m;
 
@@ -1515,7 +1515,6 @@ void vt_getmatches(struct _vtx *vx)
 		   out + pmatch[0].rm_so));
 
 	  /* check to see if this match goes across line boundaries ... */
-
 	  out += pmatch[0].rm_eo; /* skip to next sub-string */
 	}
 
@@ -1557,9 +1556,9 @@ vt_match_clear(struct _vtx *vx, char *regex)
   while (n) {
     if (regex==0 || strcmp(m->regex, regex)==0) {
       vt_list_remove((struct vt_listnode *)m);
-      free(m->regex);
+      g_free(m->regex);
       regfree(&m->preg);
-      free(m);
+      g_free(m);
     }
     m = n;
     n = n->next;
